
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/WebService/TeamCity.pm',
    'lib/WebService/TeamCity/Entity.pm',
    'lib/WebService/TeamCity/Entity/Build.pm',
    'lib/WebService/TeamCity/Entity/BuildType.pm',
    'lib/WebService/TeamCity/Entity/HasDescription.pm',
    'lib/WebService/TeamCity/Entity/HasID.pm',
    'lib/WebService/TeamCity/Entity/HasName.pm',
    'lib/WebService/TeamCity/Entity/HasStatus.pm',
    'lib/WebService/TeamCity/Entity/HasWebURL.pm',
    'lib/WebService/TeamCity/Entity/Project.pm',
    'lib/WebService/TeamCity/Entity/TestOccurrence.pm',
    'lib/WebService/TeamCity/Inflator.pm',
    'lib/WebService/TeamCity/Iterator.pm',
    'lib/WebService/TeamCity/LocatorSpec.pm',
    'lib/WebService/TeamCity/Types.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/artifacts.t',
    't/author-00-compile.t',
    't/author-eol.t',
    't/author-no-tabs.t',
    't/author-pod-spell.t',
    't/author-pod-syntax.t',
    't/author-test-version.t',
    't/basic.t',
    't/decamelize.t',
    't/fixtures/buildTypes.json',
    't/fixtures/buildTypes/id:TeamCityPluginsByJetBrains_Git_JetBrainsGitPluginTeamCity91x.json',
    't/fixtures/buildTypes/id:TeamCityPluginsByJetBrains_Git_JetBrainsGitPluginTeamCity91x/builds.json',
    't/fixtures/builds.json',
    't/fixtures/builds/id:661984.json',
    't/fixtures/builds/id:661984/artifacts/children.json',
    't/fixtures/builds/id:661984/relatedIssues.json',
    't/fixtures/builds/id:661984/statistics.json',
    't/fixtures/builds/id:666188.json',
    't/fixtures/builds/id:666188/artifacts/children.json',
    't/fixtures/builds/id:666188/relatedIssues.json',
    't/fixtures/builds/id:666188/statistics.json',
    't/fixtures/builds/id:667885.json',
    't/fixtures/builds/id:667885/artifacts/children.json',
    't/fixtures/builds/id:667885/artifacts/metadata/jetbrains.git.zip.json',
    't/fixtures/builds/id:667885/relatedIssues.json',
    't/fixtures/builds/id:667885/statistics.json',
    't/fixtures/builds?locator=count:100,start:100.json',
    't/fixtures/builds?locator=id:has-no-builds.json',
    't/fixtures/changes/id:2241127.json',
    't/fixtures/changes/id:2252908.json',
    't/fixtures/changes/id:2256115.json',
    't/fixtures/changes?locator=build:(id:661984).json',
    't/fixtures/changes?locator=build:(id:666188).json',
    't/fixtures/changes?locator=build:(id:667885).json',
    't/fixtures/generate-fixtures.pl',
    't/fixtures/projects.json',
    't/fixtures/projects/id:TeamCityPluginsByJetBrains_Git.json',
    't/fixtures/projects/id:_Root.json',
    't/fixtures/testOccurrences/id:423,build:(id:667885).json',
    't/fixtures/testOccurrences/id:426,build:(id:667885).json',
    't/fixtures/testOccurrences/id:428,build:(id:667885).json',
    't/fixtures/testOccurrences/id:430,build:(id:667885).json',
    't/fixtures/testOccurrences/id:432,build:(id:667885).json',
    't/fixtures/testOccurrences/id:434,build:(id:667885).json',
    't/fixtures/testOccurrences/id:436,build:(id:667885).json',
    't/fixtures/testOccurrences/id:438,build:(id:667885).json',
    't/fixtures/testOccurrences/id:440,build:(id:667885).json',
    't/fixtures/testOccurrences/id:444,build:(id:667885).json',
    't/fixtures/testOccurrences/id:448,build:(id:667885).json',
    't/fixtures/testOccurrences/id:450,build:(id:667885).json',
    't/fixtures/testOccurrences/id:452,build:(id:667885).json',
    't/fixtures/testOccurrences/id:454,build:(id:667885).json',
    't/fixtures/testOccurrences/id:458,build:(id:667885).json',
    't/fixtures/testOccurrences/id:460,build:(id:667885).json',
    't/fixtures/testOccurrences/id:462,build:(id:667885).json',
    't/fixtures/testOccurrences/id:466,build:(id:667885).json',
    't/fixtures/testOccurrences/id:468,build:(id:667885).json',
    't/fixtures/testOccurrences/id:470,build:(id:667885).json',
    't/fixtures/testOccurrences/id:472,build:(id:667885).json',
    't/fixtures/testOccurrences/id:476,build:(id:667885).json',
    't/fixtures/testOccurrences/id:478,build:(id:667885).json',
    't/fixtures/testOccurrences/id:480,build:(id:667885).json',
    't/fixtures/testOccurrences/id:482,build:(id:667885).json',
    't/fixtures/testOccurrences/id:484,build:(id:667885).json',
    't/fixtures/testOccurrences/id:486,build:(id:667885).json',
    't/fixtures/testOccurrences/id:490,build:(id:667885).json',
    't/fixtures/testOccurrences/id:492,build:(id:667885).json',
    't/fixtures/testOccurrences/id:499,build:(id:667885).json',
    't/fixtures/testOccurrences/id:501,build:(id:667885).json',
    't/fixtures/testOccurrences/id:503,build:(id:667885).json',
    't/fixtures/testOccurrences/id:505,build:(id:667885).json',
    't/fixtures/testOccurrences/id:507,build:(id:667885).json',
    't/fixtures/testOccurrences/id:509,build:(id:667885).json',
    't/fixtures/testOccurrences/id:511,build:(id:667885).json',
    't/fixtures/testOccurrences/id:515,build:(id:667885).json',
    't/fixtures/testOccurrences/id:519,build:(id:667885).json',
    't/fixtures/testOccurrences/id:521,build:(id:667885).json',
    't/fixtures/testOccurrences/id:523,build:(id:667885).json',
    't/fixtures/testOccurrences/id:526,build:(id:667885).json',
    't/fixtures/testOccurrences/id:530,build:(id:667885).json',
    't/fixtures/testOccurrences/id:532,build:(id:667885).json',
    't/fixtures/testOccurrences/id:534,build:(id:667885).json',
    't/fixtures/testOccurrences/id:537,build:(id:667885).json',
    't/fixtures/testOccurrences/id:540,build:(id:667885).json',
    't/fixtures/testOccurrences/id:542,build:(id:667885).json',
    't/fixtures/testOccurrences/id:544,build:(id:667885).json',
    't/fixtures/testOccurrences/id:546,build:(id:667885).json',
    't/fixtures/testOccurrences/id:548,build:(id:667885).json',
    't/fixtures/testOccurrences/id:550,build:(id:667885).json',
    't/fixtures/testOccurrences/id:552,build:(id:667885).json',
    't/fixtures/testOccurrences/id:554,build:(id:667885).json',
    't/fixtures/testOccurrences/id:556,build:(id:667885).json',
    't/fixtures/testOccurrences/id:558,build:(id:667885).json',
    't/fixtures/testOccurrences/id:562,build:(id:667885).json',
    't/fixtures/testOccurrences/id:566,build:(id:667885).json',
    't/fixtures/testOccurrences/id:574,build:(id:667885).json',
    't/fixtures/testOccurrences/id:582,build:(id:667885).json',
    't/fixtures/testOccurrences/id:588,build:(id:667885).json',
    't/fixtures/testOccurrences/id:594,build:(id:667885).json',
    't/fixtures/testOccurrences/id:596,build:(id:667885).json',
    't/fixtures/testOccurrences/id:598,build:(id:667885).json',
    't/fixtures/testOccurrences/id:600,build:(id:667885).json',
    't/fixtures/testOccurrences/id:602,build:(id:667885).json',
    't/fixtures/testOccurrences/id:604,build:(id:667885).json',
    't/fixtures/testOccurrences/id:606,build:(id:667885).json',
    't/fixtures/testOccurrences/id:608,build:(id:667885).json',
    't/fixtures/testOccurrences/id:610,build:(id:667885).json',
    't/fixtures/testOccurrences/id:612,build:(id:667885).json',
    't/fixtures/testOccurrences/id:614,build:(id:667885).json',
    't/fixtures/testOccurrences/id:616,build:(id:667885).json',
    't/fixtures/testOccurrences/id:618,build:(id:667885).json',
    't/fixtures/testOccurrences/id:620,build:(id:667885).json',
    't/fixtures/testOccurrences/id:622,build:(id:667885).json',
    't/fixtures/testOccurrences/id:652,build:(id:667885).json',
    't/fixtures/testOccurrences/id:666,build:(id:667885).json',
    't/fixtures/testOccurrences/id:669,build:(id:667885).json',
    't/fixtures/testOccurrences/id:671,build:(id:667885).json',
    't/fixtures/testOccurrences/id:673,build:(id:667885).json',
    't/fixtures/testOccurrences/id:675,build:(id:667885).json',
    't/fixtures/testOccurrences/id:677,build:(id:667885).json',
    't/fixtures/testOccurrences/id:679,build:(id:667885).json',
    't/fixtures/testOccurrences/id:681,build:(id:667885).json',
    't/fixtures/testOccurrences/id:683,build:(id:667885).json',
    't/fixtures/testOccurrences/id:685,build:(id:667885).json',
    't/fixtures/testOccurrences/id:687,build:(id:667885).json',
    't/fixtures/testOccurrences/id:689,build:(id:667885).json',
    't/fixtures/testOccurrences/id:691,build:(id:667885).json',
    't/fixtures/testOccurrences/id:693,build:(id:667885).json',
    't/fixtures/testOccurrences/id:695,build:(id:667885).json',
    't/fixtures/testOccurrences/id:697,build:(id:667885).json',
    't/fixtures/testOccurrences/id:699,build:(id:667885).json',
    't/fixtures/testOccurrences/id:701,build:(id:667885).json',
    't/fixtures/testOccurrences/id:703,build:(id:667885).json',
    't/fixtures/testOccurrences/id:705,build:(id:667885).json',
    't/fixtures/testOccurrences/id:707,build:(id:667885).json',
    't/fixtures/testOccurrences/id:709,build:(id:667885).json',
    't/fixtures/testOccurrences/id:711,build:(id:667885).json',
    't/fixtures/testOccurrences/id:713,build:(id:667885).json',
    't/fixtures/testOccurrences?locator=build:(id:667885).json',
    't/fixtures/testOccurrences?locator=count:100,start:100,build:(id:667885).json',
    't/fixtures/testOccurrences?locator=count:100,start:200,build:(id:667885).json',
    't/lib/Test/UA.pm',
    't/locator.t',
    't/release-cpan-changes.t',
    't/release-pod-coverage.t',
    't/release-tidyall.t'
);

notabs_ok($_) foreach @files;
done_testing;
